@@ -215,10 +215,15 @@ static GSList *request_list = NULL;
 static GHashTable *listener_table = NULL;
 static time_t next_refresh;
 static GHashTable *partial_tcp_req_table;
+static guint cache_timer = 0;
 
 static guint16 get_id(void)
 {
-	return random();
+	uint64_t rand;
+
+	__connman_util_get_random(&rand);
+
+	return rand;
 }
 
 static int protocol_offset(int protocol)
@@ -764,6 +769,8 @@ static void cache_element_destroy(gpointer value)
 
 static gboolean try_remove_cache(gpointer user_data)
 {
+	cache_timer = 0;
+
 	if (__sync_fetch_and_sub(&cache_refcount, 1) == 1) {
 		DBG("No cache users, removing it.");
 
@@ -2204,7 +2211,8 @@ static void destroy_server(struct server_data *server)
 	 * without any good reason. The small delay allows the new RDNSS to
 	 * create a new DNS server instance and the refcount does not go to 0.
 	 */
-	g_timeout_add_seconds(3, try_remove_cache, NULL);
+	if (cache && !cache_timer)
+		cache_timer = g_timeout_add_seconds(3, try_remove_cache, NULL);
 
 	g_free(server);
 }
@@ -3828,8 +3836,6 @@ int __connman_dnsproxy_init(void)
 
 	DBG("");
 
-	srandom(time(NULL));
-
 	listener_table = g_hash_table_new_full(g_direct_hash, g_direct_equal,
 							NULL, g_free);
 
@@ -3861,6 +3867,16 @@ void __connman_dnsproxy_cleanup(void)
 {
 	DBG("");
 
+	if (cache_timer) {
+		g_source_remove(cache_timer);
+		cache_timer = 0;
+	}
+
+	if (cache) {
+		g_hash_table_destroy(cache);
+		cache = NULL;
+	}
+
 	connman_notifier_unregister(&dnsproxy_notifier);
 
 	g_hash_table_foreach(listener_table, remove_listener, NULL);
