@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -56,6 +56,7 @@ static char *default_blacklist[] = {
 	"vmnet",
 	"vboxnet",
 	"virbr",
+	"ifb",
 	NULL
 };
 
@@ -602,12 +603,6 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (mkdir(STATEDIR, S_IRUSR | S_IWUSR | S_IXUSR |
-				S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0) {
-		if (errno != EEXIST)
-			perror("Failed to create state directory");
-	}
-
 	if (mkdir(STORAGEDIR, S_IRUSR | S_IWUSR | S_IXUSR |
 				S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0) {
 		if (errno != EEXIST)
@@ -644,11 +639,14 @@ int main(int argc, char *argv[])
 	else
 		config_init(option_config);
 
+	__connman_util_init();
 	__connman_inotify_init();
 	__connman_technology_init();
 	__connman_notifier_init();
 	__connman_agent_init();
 	__connman_service_init();
+	__connman_peer_service_init();
+	__connman_peer_init();
 	__connman_provider_init();
 	__connman_network_init();
 	__connman_config_init();
@@ -682,6 +680,7 @@ int main(int argc, char *argv[])
 	__connman_wpad_init();
 	__connman_wispr_init();
 	__connman_rfkill_init();
+	__connman_machine_init();
 
 	g_free(option_config);
 	g_free(option_device);
@@ -693,11 +692,11 @@ int main(int argc, char *argv[])
 
 	g_source_remove(signal);
 
+	__connman_machine_cleanup();
 	__connman_rfkill_cleanup();
 	__connman_wispr_cleanup();
 	__connman_wpad_cleanup();
 	__connman_dhcpv6_cleanup();
-	__connman_dhcp_cleanup();
 	__connman_session_cleanup();
 	__connman_plugin_cleanup();
 	__connman_provider_cleanup();
@@ -717,11 +716,13 @@ int main(int argc, char *argv[])
 	__connman_tethering_cleanup();
 	__connman_nat_cleanup();
 	__connman_firewall_cleanup();
-	__connman_nfacct_cleanup();
 	__connman_iptables_cleanup();
+	__connman_peer_service_cleanup();
+	__connman_peer_cleanup();
 	__connman_ippool_cleanup();
 	__connman_device_cleanup();
 	__connman_network_cleanup();
+	__connman_dhcp_cleanup();
 	__connman_service_cleanup();
 	__connman_agent_cleanup();
 	__connman_ipconfig_cleanup();
@@ -729,6 +730,7 @@ int main(int argc, char *argv[])
 	__connman_technology_cleanup();
 	__connman_inotify_cleanup();
 
+	__connman_util_cleanup();
 	__connman_dbus_cleanup();
 
 	__connman_log_cleanup(option_backtrace);
@@ -747,6 +749,7 @@ int main(int argc, char *argv[])
 	g_strfreev(connman_settings.tethering_technologies);
 
 	g_free(option_debug);
+	g_free(option_wifi);
 
 	return 0;
 }
